// $ANTLR 2.7.2: "eval.g" -> "StringTemplateLanguageEvaluator.cs"$

/*
 [The "BSD licence"]
 Copyright (c) 2003 Terence Parr
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using Antlr.StringTemplate;
using System.Collections;
using System.IO;

namespace Antlr.StringTemplate.Language
{
	// Generate header specific to the tree-parser CSharp file
	using System;
	
	using TreeParser = antlr.TreeParser;
	using Token                    = antlr.Token;
	using AST                      = antlr.collections.AST;
	using RecognitionException     = antlr.RecognitionException;
	using ANTLRException           = antlr.ANTLRException;
	using NoViableAltException     = antlr.NoViableAltException;
	using MismatchedTokenException = antlr.MismatchedTokenException;
	using SemanticException        = antlr.SemanticException;
	using BitSet                   = antlr.collections.impl.BitSet;
	using ASTPair                  = antlr.ASTPair;
	using ASTFactory               = antlr.ASTFactory;
	using ASTArray                 = antlr.collections.impl.ASTArray;
	
	
	public 	class StringTemplateLanguageEvaluator : antlr.TreeParser
	{
		public const int EOF = 1;
		public const int NULL_TREE_LOOKAHEAD = 3;
		public const int APPLY = 4;
		public const int ARGS = 5;
		public const int INCLUDE = 6;
		public const int CONDITIONAL = 7;
		public const int VALUE = 8;
		public const int TEMPLATE = 9;
		public const int SEMI = 10;
		public const int LPAREN = 11;
		public const int RPAREN = 12;
		public const int LITERAL_separator = 13;
		public const int ASSIGN = 14;
		public const int NOT = 15;
		public const int PLUS = 16;
		public const int COLON = 17;
		public const int COMMA = 18;
		public const int ID = 19;
		public const int ANONYMOUS_TEMPLATE = 20;
		public const int STRING = 21;
		public const int INT = 22;
		public const int DOT = 23;
		public const int ESC_CHAR = 24;
		public const int WS = 25;
		
		
    public class NameValuePair {
        public String name;
        public Object value;
    };

    protected StringTemplate self = null;
    protected TextWriter outWriter = null;
    protected ASTExpr chunk = null;

    /** Create an evaluator using attributes from self */
    public StringTemplateLanguageEvaluator(StringTemplate self, ASTExpr chunk, TextWriter outWriter) {
        this.self = self;
        this.chunk = chunk;
        this.outWriter = outWriter;
    }

		public StringTemplateLanguageEvaluator()
		{
			tokenNames = tokenNames_;
		}
		
	public void action(AST _t) //throws RecognitionException
{
		
		StringTemplateAST action_AST_in = (StringTemplateAST)_t;
		
		Object e=null;
		
		
		try {      // for error handling
			e=expr(_t);
			_t = retTree_;
			chunk.WriteAttribute(self, e, outWriter);
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
	}
	
	public Object  expr(AST _t) //throws RecognitionException
{
		Object value=null;
		
		StringTemplateAST expr_AST_in = (StringTemplateAST)_t;
		
		Object a=null, b=null, e=null;
		Hashtable argumentContext=null;
		
		
		try {      // for error handling
			if (null == _t)
				_t = ASTNULL;
			switch ( _t.Type )
			{
			case PLUS:
			{
				AST __t3 = _t;
				StringTemplateAST tmp1_AST_in = (StringTemplateAST)_t;
				match((AST)_t,PLUS);
				_t = _t.getFirstChild();
				a=expr(_t);
				_t = retTree_;
				b=expr(_t);
				_t = retTree_;
				value = chunk.Add(a, b);
				_t = __t3;
				_t = _t.getNextSibling();
				break;
			}
			case APPLY:
			{
				value=templateApplication(_t);
				_t = retTree_;
				break;
			}
			case ID:
			case STRING:
			case INT:
			case DOT:
			{
				value=attribute(_t);
				_t = retTree_;
				break;
			}
			case INCLUDE:
			{
				value=templateInclude(_t);
				_t = retTree_;
				break;
			}
			case VALUE:
			{
				AST __t4 = _t;
				StringTemplateAST tmp2_AST_in = (StringTemplateAST)_t;
				match((AST)_t,VALUE);
				_t = _t.getFirstChild();
				e=expr(_t);
				_t = retTree_;
				_t = __t4;
				_t = _t.getNextSibling();
				
				StringWriter buf = new StringWriter();
				chunk.WriteAttribute(self, e,buf);
				value = buf.ToString();
				
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			 }
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
/** Apply template(s) to an attribute; can be applied to another apply
 *  result.  I am not too efficient here as I write to a StringWriter and
 *  then dump for the common <attribute:template()> case, but I'll fix later.
 */
	public Object  templateApplication(AST _t) //throws RecognitionException
{
		Object value=null;
		
		StringTemplateAST templateApplication_AST_in = (StringTemplateAST)_t;
		
		Object a=null;
		ArrayList templatesToApply=new ArrayList();
		
		
		try {      // for error handling
			AST __t9 = _t;
			StringTemplateAST tmp3_AST_in = (StringTemplateAST)_t;
			match((AST)_t,APPLY);
			_t = _t.getFirstChild();
			a=expr(_t);
			_t = retTree_;
			{ // ( ... )+
			int _cnt11=0;
			for (;;)
			{
				if (_t == null)
					_t = ASTNULL;
				if ((_t.Type==TEMPLATE))
				{
					template(_t,templatesToApply);
					_t = retTree_;
				}
				else
				{
					if (_cnt11 >= 1) { goto _loop11_breakloop; } else { throw new NoViableAltException(_t);; }
				}
				
				_cnt11++;
			}
_loop11_breakloop:			;
			}    // ( ... )+
			value = chunk.ApplyListOfAlternatingTemplates(self, a,templatesToApply);
			_t = __t9;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
	public Object  attribute(AST _t) //throws RecognitionException
{
		Object value=null;
		
		StringTemplateAST attribute_AST_in = (StringTemplateAST)_t;
		StringTemplateAST prop = null;
		StringTemplateAST i3 = null;
		StringTemplateAST i = null;
		StringTemplateAST s = null;
		
		Object obj = null;
		
		
		try {      // for error handling
			if (null == _t)
				_t = ASTNULL;
			switch ( _t.Type )
			{
			case DOT:
			{
				AST __t19 = _t;
				StringTemplateAST tmp4_AST_in = (StringTemplateAST)_t;
				match((AST)_t,DOT);
				_t = _t.getFirstChild();
				obj=attribute(_t);
				_t = retTree_;
				prop = (StringTemplateAST)_t;
				match((AST)_t,ID);
				_t = _t.getNextSibling();
				_t = __t19;
				_t = _t.getNextSibling();
				value = chunk.GetObjectProperty(self, obj, prop.getText());
				break;
			}
			case ID:
			{
				i3 = (StringTemplateAST)_t;
				match((AST)_t,ID);
				_t = _t.getNextSibling();
				value=self.GetAttribute(i3.getText());
				break;
			}
			case INT:
			{
				i = (StringTemplateAST)_t;
				match((AST)_t,INT);
				_t = _t.getNextSibling();
				value = Int32.Parse(i.getText());
				break;
			}
			case STRING:
			{
				s = (StringTemplateAST)_t;
				match((AST)_t,STRING);
				_t = _t.getNextSibling();
				value=s.getText();
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			 }
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
	public Object  templateInclude(AST _t) //throws RecognitionException
{
		Object value=null;
		
		StringTemplateAST templateInclude_AST_in = (StringTemplateAST)_t;
		StringTemplateAST t2 = null;
		
		Hashtable argumentContext = null;
		String name = null;
		Object n=null;
		
		
		try {      // for error handling
			AST __t6 = _t;
			StringTemplateAST tmp5_AST_in = (StringTemplateAST)_t;
			match((AST)_t,INCLUDE);
			_t = _t.getFirstChild();
			{
				t2 = (StringTemplateAST)_t;
				match((AST)_t,ID);
				_t = _t.getNextSibling();
				argumentContext=argList(_t,null);
				_t = retTree_;
				name=t2.getText();
			}
			_t = __t6;
			_t = _t.getNextSibling();
			value = chunk.GetTemplateText(self, name, argumentContext);
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
	public Hashtable  argList(AST _t,
		Hashtable initialContext
	) //throws RecognitionException
{
		Hashtable argumentContext=null;
		
		StringTemplateAST argList_AST_in = (StringTemplateAST)_t;
		
		argumentContext = initialContext;
		if ( argumentContext == null ) {
		argumentContext=new Hashtable();
		}
		
		
		try {      // for error handling
			AST __t21 = _t;
			StringTemplateAST tmp6_AST_in = (StringTemplateAST)_t;
			match((AST)_t,ARGS);
			_t = _t.getFirstChild();
			{    // ( ... )*
				for (;;)
				{
					if (_t == null)
						_t = ASTNULL;
					if ((_t.Type==ASSIGN))
					{
						argumentAssignment(_t,argumentContext);
						_t = retTree_;
					}
					else
					{
						goto _loop23_breakloop;
					}
					
				}
_loop23_breakloop:				;
			}    // ( ... )*
			_t = __t21;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return argumentContext;
	}
	
	public void template(AST _t,
		ArrayList templatesToApply
	) //throws RecognitionException
{
		
		StringTemplateAST template_AST_in = (StringTemplateAST)_t;
		StringTemplateAST t = null;
		StringTemplateAST args = null;
		StringTemplateAST anon = null;
		
		Hashtable argumentContext = null;
		Object n = null;
		
		
		try {      // for error handling
			AST __t13 = _t;
			StringTemplateAST tmp7_AST_in = (StringTemplateAST)_t;
			match((AST)_t,TEMPLATE);
			_t = _t.getFirstChild();
			{
				if (null == _t)
					_t = ASTNULL;
				switch ( _t.Type )
				{
				case ID:
				{
					t = (StringTemplateAST)_t;
					match((AST)_t,ID);
					_t = _t.getNextSibling();
					args = (StringTemplateAST)_t;
					if (null == _t) throw new MismatchedTokenException();
					_t = _t.getNextSibling();
					
					String templateName = t.getText();
					StringTemplateGroup group = self.Group;
					StringTemplate embedded = group.GetEmbeddedInstanceOf(self, templateName);
					if ( embedded != null ) {
					embedded.ArgumentsAST = args;
					templatesToApply.Add(embedded);
					}
					
					break;
				}
				case ANONYMOUS_TEMPLATE:
				{
					anon = (StringTemplateAST)_t;
					match((AST)_t,ANONYMOUS_TEMPLATE);
					_t = _t.getNextSibling();
					
					StringTemplate anonymous = anon.StringTemplate;
					//anonymous.setArgumentContext(new Hashtable());
					templatesToApply.Add(anonymous);
					
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				 }
			}
			_t = __t13;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
	}
	
	public bool  ifCondition(AST _t) //throws RecognitionException
{
		bool value=false;
		
		StringTemplateAST ifCondition_AST_in = (StringTemplateAST)_t;
		
		Object a=null, b=null;
		
		
		try {      // for error handling
			if (null == _t)
				_t = ASTNULL;
			switch ( _t.Type )
			{
			case APPLY:
			case INCLUDE:
			case VALUE:
			case PLUS:
			case ID:
			case STRING:
			case INT:
			case DOT:
			{
				a=ifAtom(_t);
				_t = retTree_;
				value = a != null;
				break;
			}
			case NOT:
			{
				AST __t16 = _t;
				StringTemplateAST tmp8_AST_in = (StringTemplateAST)_t;
				match((AST)_t,NOT);
				_t = _t.getFirstChild();
				a=ifAtom(_t);
				_t = retTree_;
				_t = __t16;
				_t = _t.getNextSibling();
				value = a == null;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			 }
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
	public Object  ifAtom(AST _t) //throws RecognitionException
{
		Object value=null;
		
		StringTemplateAST ifAtom_AST_in = (StringTemplateAST)_t;
		
		try {      // for error handling
			value=expr(_t);
			_t = retTree_;
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
		return value;
	}
	
	public void argumentAssignment(AST _t,
		Hashtable argumentContext
	) //throws RecognitionException
{
		
		StringTemplateAST argumentAssignment_AST_in = (StringTemplateAST)_t;
		StringTemplateAST arg = null;
		
		Object e = null;
		
		
		try {      // for error handling
			AST __t25 = _t;
			StringTemplateAST tmp9_AST_in = (StringTemplateAST)_t;
			match((AST)_t,ASSIGN);
			_t = _t.getFirstChild();
			arg = (StringTemplateAST)_t;
			match((AST)_t,ID);
			_t = _t.getNextSibling();
			e=expr(_t);
			_t = retTree_;
			if ( e != null ) argumentContext[arg.getText()] = e;
			_t = __t25;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex)
		{
			reportError(ex);
			if (null != _t)
			{
				_t = _t.getNextSibling();
			}
		}
		retTree_ = _t;
	}
	
	public new StringTemplateAST getAST()
	{
		return (StringTemplateAST) returnAST;
	}
	
	
	public static readonly string[] tokenNames_ = new string[] {
		@"""<0>""",
		@"""EOF""",
		@"""<2>""",
		@"""NULL_TREE_LOOKAHEAD""",
		@"""APPLY""",
		@"""ARGS""",
		@"""INCLUDE""",
		@"""if""",
		@"""VALUE""",
		@"""TEMPLATE""",
		@"""SEMI""",
		@"""LPAREN""",
		@"""RPAREN""",
		@"""separator""",
		@"""ASSIGN""",
		@"""NOT""",
		@"""PLUS""",
		@"""COLON""",
		@"""COMMA""",
		@"""ID""",
		@"""ANONYMOUS_TEMPLATE""",
		@"""STRING""",
		@"""INT""",
		@"""DOT""",
		@"""ESC_CHAR""",
		@"""WS"""
	};
	
}

}
